/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file SystemError.h
 *    A platform independent way to get the last system error code.
 *
 * @author Christian Martin
 * @date   2011/05/05
 */

#ifndef _MIRA_SYSTEM_ERROR_H_
#define _MIRA_SYSTEM_ERROR_H_

#ifndef Q_MOC_RUN
#include <boost/system/error_code.hpp>
#endif

#include <platform/Platform.h>

#if defined(MIRA_LINUX)
# include <errno.h>
#elif defined(MIRA_WINDOWS)
#endif

namespace mira {

//////////////////////////////////////////////////////////////////////////////

#if defined(MIRA_LINUX)

/// Return the last system error code.
inline boost::system::error_code getLastSystemErrorCode()
{
	return boost::system::error_code(errno,
#if BOOST_VERSION < 105500
	                                 boost::system::get_generic_category());
#else
	                                 boost::system::generic_category());
#endif
}

/// Set a new last system error code
inline void setLastSystemErrorCode(int errorCode)
{
	errno = errorCode;
}

#elif defined(MIRA_WINDOWS)

/// Return the last system error code.
inline boost::system::error_code getLastSystemErrorCode()
{
	return boost::system::error_code(GetLastError(),
	                                 boost::system::get_system_category());
}

/// Set a new last system error code
inline void setLastSystemErrorCode(int errorCode)
{
	SetLastError(errorCode);
}

#endif

//////////////////////////////////////////////////////////////////////////////

}

#endif
