/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file Console.h
 *    Platform dependent functions to control the console and its settings.
 *
 * @author Erik Einhorn
 * @date   2011/02/20
 */

#ifndef _MIRA_CONSOLE_H_
#define _MIRA_CONSOLE_H_

#include <platform/Platform.h>

namespace mira { namespace console {

///////////////////////////////////////////////////////////////////////////////

/**
 * Enables or disables echo for text that is typed when using std input, e.g.
 * cin.
 * Disabling the echo is useful when the user types passwords, etc.
 */
MIRA_BASE_EXPORT void enableEcho(bool enable = true);

enum TextColor
{
	DEFAULT = 0,

	BLACK,
	DARKGREY,
	LIGHTGREY,
	WHITE,

	RED,
	GREEN,
	YELLOW,
	BLUE,
	MAGENTA,
	CYAN,

	LIGHTRED,
	LIGHTGREEN,
	LIGHTYELLOW,
	LIGHTBLUE,
	LIGHTMAGENTA,
	LIGHTCYAN,
};

enum StdOutput
{
	STD_OUTPUT,	///< std::cout
	STD_ERROR	///< std::cerr
};

/**
 * Sets the color of the console text as specified.
 * @param[in] color The color for the console text
 * @param[in] output The console where the color change should be applied
 */
MIRA_BASE_EXPORT void setTextColor(TextColor color = DEFAULT,
                                   StdOutput output = STD_OUTPUT);


/**
 * Returns the number of columns of the console, i.e. the width of the text
 * area.
 */
MIRA_BASE_EXPORT int getWidth();


///////////////////////////////////////////////////////////////////////////////

}} // namespaces

#endif /* _MIRA_CONSOLE_H_ */
