/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file GridObject.C
 *    Implementation of GridObject.
 *
 * @author Tim Langner
 * @date   2011/01/06
 */

#include <OGRE/OgreSceneManager.h>
#include <OGRE/OgreSceneNode.h>
#include <OGRE/OgreManualObject.h>
#include <OGRE/OgreMaterialManager.h>

#include <visualization/3d/GridObject.h>

namespace mira {

///////////////////////////////////////////////////////////////////////////////

GridObject::GridObject(Ogre::SceneManager* sceneManager, Ogre::SceneNode* parent) :
	VisualizationObject(sceneManager, parent),
	mColor(Ogre::ColourValue::Black),
	mCellCount(20),
	mCellSize(0.2f)
{
	mManualObject = mSceneManager->createManualObject("GridObject"+ toString(this));
	mManualObject->setCastShadows(false);
	mNode->attachObject(mManualObject);
	setColor(mColor);
	setupGrid();
}

GridObject::~GridObject()
{
	mNode->detachObject(mManualObject);
	mSceneManager->destroyManualObject(mManualObject);
}

void GridObject::setColor(const Ogre::ColourValue& color)
{
	mColor = color;
	setupGrid();
}

void GridObject::setCellCount(int count)
{
	mCellCount = count;
	setupGrid();
}

void GridObject::setCellSize(float size)
{
	mCellSize = size;
	setupGrid();
}

void GridObject::setupGrid()
{
	mManualObject->clear();
	float extent = (mCellSize*((float)mCellCount))/2;
	mManualObject->estimateVertexCount(mCellCount * 4);

	mManualObject->begin("TransparentNoLight", Ogre::RenderOperation::OT_LINE_LIST);

	for (int i = 0; i <= mCellCount; i++)
	{
		float inc = extent - (i * mCellSize);

		Ogre::Vector3 p1(inc, -extent, 0);
		Ogre::Vector3 p2(inc, extent, 0);
		Ogre::Vector3 p3(-extent, inc, 0);
		Ogre::Vector3 p4(extent, inc, 0);

		mManualObject->position(p1);
		mManualObject->colour(mColor);
		mManualObject->position(p2);
		mManualObject->colour(mColor);
		mManualObject->position(p3);
		mManualObject->colour(mColor);
		mManualObject->position(p4);
		mManualObject->colour(mColor);
	}
	mManualObject->end();
}

///////////////////////////////////////////////////////////////////////////////

}
