/*
 * Copyright (C) 2012 by
 *   MetraLabs GmbH (MLAB), GERMANY
 * and
 *   Neuroinformatics and Cognitive Robotics Labs (NICR) at TU Ilmenau, GERMANY
 * All rights reserved.
 *
 * Contact: info@mira-project.org
 *
 * Commercial Usage:
 *   Licensees holding valid commercial licenses may use this file in
 *   accordance with the commercial license agreement provided with the
 *   software or, alternatively, in accordance with the terms contained in
 *   a written agreement between you and MLAB or NICR.
 *
 * GNU General Public License Usage:
 *   Alternatively, this file may be used under the terms of the GNU
 *   General Public License version 3.0 as published by the Free Software
 *   Foundation and appearing in the file LICENSE.GPL3 included in the
 *   packaging of this file. Please review the following information to
 *   ensure the GNU General Public License version 3.0 requirements will be
 *   met: http://www.gnu.org/copyleft/gpl.html.
 *   Alternatively you may (at your option) use any later version of the GNU
 *   General Public License if such license has been publicly approved by
 *   MLAB and NICR (or its successors, if any).
 *
 * IN NO EVENT SHALL "MLAB" OR "NICR" BE LIABLE TO ANY PARTY FOR DIRECT,
 * INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING OUT OF
 * THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF "MLAB" OR
 * "NICR" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * "MLAB" AND "NICR" SPECIFICALLY DISCLAIM ANY WARRANTIES, INCLUDING,
 * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE PROVIDED HEREUNDER IS
 * ON AN "AS IS" BASIS, AND "MLAB" AND "NICR" HAVE NO OBLIGATION TO
 * PROVIDE MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS OR MODIFICATIONS.
 */

/**
 * @file IDrive.h
 *    An interface class for objects, which provide a "drive" capability.
 *
 * @author Christian Martin
 * @date   2012/02/27
 */

#ifndef _MIRA_IDRIVE_H_
#define _MIRA_IDRIVE_H_

#include <transform/Velocity.h>

namespace mira { namespace robot {

///////////////////////////////////////////////////////////////////////////////

/// An interface class for objects, which provide a "drive" capability.
class IDrive
{
public:
	/** @name Destructor and reflection */
	//@{

	template<typename Reflector>
	void reflect(Reflector& r)
	{
		r.interface("IDrive");
		r.method("setVelocity", &IDrive::setVelocity, this,
		         "Set translational and rotational velocity.",
		         "v", "velocity", Velocity2(0.5, 0.0, 0.0));
	}

	virtual ~IDrive() {}

	//@}

public:
	/** @name IDrive interface definition */
	//@{

	/**
	 * Set translational and rotational velocity.
	 * @param[in] v The translation and rotational velocity as a Velocity2.
	 */
	virtual void setVelocity(const Velocity2& v) = 0;

	//@}
};

///////////////////////////////////////////////////////////////////////////////

}} // namespace

#endif
